import functools
import warnings
from datetime import timezone

import spotipy
from dateutil.parser import parse as datetime_parse
from feedgen.feed import FeedGenerator
from spotipy.oauth2 import SpotifyClientCredentials


def ignore_spotify_errors(func):
    @functools.wraps(func)
    def wrapper(*args, **kwargs):
        try:
            return func(*args, **kwargs)
        except spotipy.SpotifyException:
            return None
    return wrapper


class SPR:
    def __init__(self, client_id, client_secret, market=None):
        self.client_id = client_id
        self.client_secret = client_secret
        self.market = market

        self.spotipy = spotipy.Spotify(
            auth_manager=SpotifyClientCredentials(client_id=client_id, client_secret=client_secret))

    @ignore_spotify_errors
    def get_episodes_by_show_id(self, show_id):
        eps = []
        offset = 0
        while True:
            ret = self.spotipy.show_episodes(show_id, limit=50, market=self.market, offset=offset)
            eps.extend([ep for ep in ret["items"] if ep["is_playable"]])
            if ret["next"] is None:
                break
            else:
                offset += 50
        return sorted(eps, key=lambda v: v.get("release_date", ""), reverse=True)

    @ignore_spotify_errors
    def get_show_by_show_id(self, show_id):
        return self.spotipy.show(show_id, market=self.market)

    def get_rss_by_show_id(self, show_id):
        show = self.get_show_by_show_id(show_id)
        if show is None:
            warnings.warn("get_show_by_show_id({}) returned None".format(show_id))
            return None
        eps = self.get_episodes_by_show_id(show_id)
        if eps is None:
            warnings.warn("get_episodes_by_show_id({}) returned None".format(show_id))

        fg = FeedGenerator()
        fg.load_extension("podcast")
        fg.id(show_id)
        fg.title(show["name"])
        if "href" in show:
            fg.link(href=show["href"], rel="via")
        if "images" in show and show["images"]:
            fg.logo(show["images"][0]["url"])
        if "languages" in show and show["languages"]:
            fg.language(show["languages"][0])
        fg.description(show.get("description", None))

        for ep in eps:
            if not ep["external_urls"]:
                continue
            url = None
            for key, value in ep["external_urls"].items():
                if key == "spotify":
                    url = value
                    break
            fe = fg.add_entry(order="append")
            fe.title(ep["name"])
            fe.id(ep["id"])
            fe.description(ep["description"])
            if len(ep["languages"]) == 1:
                fe.link(href=url, hreflang=ep["languages"][0])
            else:
                fe.link(href=url)
            if ep["explicit"]:
                fe.podcast.itunes_explicit("yes")
            if ep["images"]:
                # Spotify says the largest image will be first
                # We do a very ugly hack here because feedgen insists that the
                # string ends with '.jpg' or '.png'
                fe.podcast.itunes_image(ep["images"][0]["url"] + "#.jpg")
            try:
                published = datetime_parse(ep["release_date"])
                if published.tzinfo is None:
                    published = published.astimezone(timezone.utc)
                fe.published(published)
            except Exception:
                pass
            fe.podcast.itunes_duration(int(ep["duration_ms"] / 1000))
        return fg.rss_str(pretty=True)



try:
    from spr.config import Config
    from spr.engine import SPR
except ImportError:
    pass

__version__ = "0.3.3"
VERSION = tuple(map(int, __version__.split(".")))

__all__ = ["SPR", "Config"]



#!/usr/bin/env python3

import argparse

from spr import SPR, Config


def main():
    parser = argparse.ArgumentParser()
    parser.add_argument("show_id", help="Spotify ID of a podcast")
    parser.add_argument("-c", "--config", help="Path to config file")
    args = parser.parse_args()

    config = Config.load(args.config)

    spr = SPR(**config.__dict__)

    print(spr.get_rss_by_show_id(args.show_id).decode())


if __name__ == "__main__":
    main()



import os
from html import escape
from urllib.parse import parse_qs, urljoin

from spr import SPR, Config


def application(environ, start_response):
    # http://wsgi.tutorial.codepoint.net/application-interface
    # https://www.toptal.com/python/pythons-wsgi-server-application-interface

    config = Config.load()
    spr = SPR(**config.__dict__)

    if environ.get("REQUEST_METHOD").upper() == "POST":
        request_body = environ["wsgi.input"].read(-1).decode("utf-8")
        post = parse_qs(request_body)
        try:
            print(post, post.get("url_or_show_id", []))
            show_id = escape(post.get("url_or_show_id", [])[-1]).strip("/").split("/")[-1]
        except IndexError:
            show_id = ""
        start_response("303 See Other", [("Location", urljoin(environ.get("REQUEST_URI"), show_id))])
        return []

    show_id = environ.get("PATH_INFO").split("/")[-1]

    if not show_id:
        with open(os.path.join(os.path.dirname(__file__), "html/index.html")) as html:
            content = html.read().encode()
        content_type = "text/html; charset=utf-8"
        status = "200 OK"

    if show_id:  # not 'else', because show_id could have been reset above
        rss = spr.get_rss_by_show_id(show_id)
        if rss is None:
            content = "Show ID {} not found".format(show_id).encode()
            content_type = "text/plain; charset=utf-8"
            status = "404 Not Found"
        else:
            content = rss
            content_type = "application/rss+xml; charset=utf-8"
            status = "200 OK"

    # Debug stuff
    # content = "\n".join(["%s: %s" % (k, v) for k, v in sorted(environ.items())]).encode()
    # status = "200 OK"
    # content_type = "text/plain; charset=utf-8"

    response_headers = [
        ("Content-Type", content_type),
        ("Content-Length", str(len(content))),
    ]

    start_response(status, response_headers)

    if environ.get("REQUEST_METHOD").upper() == "HEAD":
        return []

    return [content]



import configparser
import os


class Config:
    def __init__(self, client_id=None, client_secret=None, market=None):
        self.client_id = client_id
        self.client_secret = client_secret
        self.market = market

    @classmethod
    def from_file(cls, filename):
        config = configparser.ConfigParser()
        config.read(filename)
        assert "spotify" in config, "No `spotify` section in {}".format(filename)
        assert "client_id" in config["spotify"], "No Spotify client ID in {}".format(filename)
        assert "client_secret" in config["spotify"], "No Spotify client secret in {}".format(filename)

        return cls(
            client_id=config["spotify"]["client_id"],
            client_secret=config["spotify"]["client_secret"],
            market=config["spotify"].get("market", None))

    @classmethod
    def load(cls, filename=None):
        if filename is None:
            filename = os.environ.get("SPR_CONFIG", "config.ini")
        return cls.from_file(filename)
